@@ -1,73 +1,42 @@
 package Net::SAML2::Protocol::LogoutResponse;
 use Moose;
-use MooseX::Types::Moose qw/ Str /;
 use MooseX::Types::URI qw/ Uri /;
+use Net::SAML2::XML::Util qw/ no_comments /;
 
 with 'Net::SAML2::Role::ProtocolMessage';
 
-=head1 NAME
+# ABSTRACT: SAML2 LogoutResponse Protocol object
 
-Net::SAML2::Protocol::LogoutResponse - the SAML2 LogoutResponse object
+our $VERSION = '0.44';
 
-=head1 SYNOPSIS
 
-  my $logout_req = Net::SAML2::Protocol::LogoutResponse->new(
-    issuer      => $issuer,
-    destination => $destination,
-    status      => $status,
-    response_to => $response_to,
-  );
+has 'status'      => (isa => 'Str', is => 'ro', required => 1);
+has 'substatus'   => (isa => 'Str', is => 'ro', required => 0);
+has 'response_to' => (isa => 'Str', is => 'ro', required => 1);
 
-=head1 METHODS
-
-=head2 new( ... )
-
-Constructor. Returns an instance of the LogoutResponse object.
-
-Arguments:
-
- * issuer - the SP's identity URI
- * destination -  the IdP's identity URI
- * status - the response status
- * response_to - the request ID we're responding to
-
-=cut
-
-has 'issuer'      => (isa => Uri, is => 'ro', required => 1, coerce => 1);
-has 'destination' => (isa => Uri, is => 'ro', required => 1, coerce => 1);
-has 'status'      => (isa => Str, is => 'ro', required => 1);
-has 'response_to' => (isa => Str, is => 'ro', required => 1);
-
-=head2 new_from_xml
-
-Create a LogoutResponse object from the given XML.
-
-=cut
 
 sub new_from_xml {
     my ($class, %args) = @_;
-     
-    my $xpath = XML::XPath->new( xml => $args{xml} );
-    $xpath->set_namespace('saml', 'urn:oasis:names:tc:SAML:2.0:assertion');
-    $xpath->set_namespace('samlp', 'urn:oasis:names:tc:SAML:2.0:protocol');
+
+    my $dom = no_comments($args{xml});
+
+    my $xpath = XML::LibXML::XPathContext->new($dom);
+    $xpath->registerNs('saml', 'urn:oasis:names:tc:SAML:2.0:assertion');
+    $xpath->registerNs('samlp', 'urn:oasis:names:tc:SAML:2.0:protocol');
 
     my $self = $class->new(
-        id          => $xpath->findvalue('/samlp:LogoutResponse/@ID')->value,
-        response_to => $xpath->findvalue('/samlp:LogoutResponse/@InResponseTo')->value,
-        destination => $xpath->findvalue('/samlp:LogoutResponse/@Destination')->value,
-        session     => $xpath->findvalue('/samlp:LogoutResponse/samlp:SessionIndex')->value,
-        issuer      => $xpath->findvalue('/samlp:LogoutResponse/saml:Issuer')->value,
-        status      => $xpath->findvalue('/samlp:LogoutResponse/samlp:Status/samlp:StatusCode/@Value')->value,
+        id          => $xpath->findvalue('/samlp:LogoutResponse/@ID'),
+        response_to => $xpath->findvalue('/samlp:LogoutResponse/@InResponseTo'),
+        destination => $xpath->findvalue('/samlp:LogoutResponse/@Destination'),
+        session     => $xpath->findvalue('/samlp:LogoutResponse/samlp:SessionIndex'),
+        issuer      => $xpath->findvalue('/samlp:LogoutResponse/saml:Issuer'),
+        status      => $xpath->findvalue('/samlp:LogoutResponse/samlp:Status/samlp:StatusCode/@Value'),
+        substatus   => $xpath->findvalue('/samlp:LogoutResponse/samlp:Status/samlp:StatusCode/samlp:StatusCode/@Value'),
     );
 
     return $self;
 }
 
-=head2 as_xml()
-
-Returns the LogoutResponse as XML.
-
-=cut
 
 sub as_xml {
     my ($self) = @_;
@@ -99,11 +68,6 @@ sub as_xml {
     );
 }
 
-=head2 success
-
-Returns true if the Response's status is Success.
-
-=cut
 
 sub success {
     my ($self) = @_;
@@ -112,3 +76,93 @@ sub success {
 }
 
 __PACKAGE__->meta->make_immutable;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Net::SAML2::Protocol::LogoutResponse - SAML2 LogoutResponse Protocol object
+
+=head1 VERSION
+
+version 0.44
+
+=head1 SYNOPSIS
+
+  my $logout_req = Net::SAML2::Protocol::LogoutResponse->new(
+    issuer      => $issuer,
+    destination => $destination,
+    status      => $status,
+    response_to => $response_to,
+  );
+
+=head1 NAME
+
+Net::SAML2::Protocol::LogoutResponse - the SAML2 LogoutResponse object
+
+=head1 METHODS
+
+=head2 new( ... )
+
+Constructor. Returns an instance of the LogoutResponse object.
+
+Arguments:
+
+=over
+
+=item B<issuer>
+
+SP's identity URI
+
+=item B<destination>
+
+IdP's identity URI
+
+=item B<status>
+
+response status
+
+=item B<response_to>
+
+request ID we're responding to
+
+=back
+
+=head2 new_from_xml( ... )
+
+Create a LogoutResponse object from the given XML.
+
+Arguments:
+
+=over
+
+=item B<xml>
+
+XML data
+
+=back
+
+=head2 as_xml( )
+
+Returns the LogoutResponse as XML.
+
+=head2 success( )
+
+Returns true if the Response's status is Success.
+
+=head1 AUTHOR
+
+Chris Andrews  <chrisa@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2021 by Chris Andrews and Others, see the git log.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
