@@ -1,54 +1,45 @@
 package Net::SAML2::SP;
 use Moose;
-use MooseX::Types::Moose qw/ Str /;
 use MooseX::Types::URI qw/ Uri /;
 
-=head1 NAME
+our $VERSION = '0.44';
 
-Net::SAML2::SP - SAML Service Provider object
+# ABSTRACT: Net::SAML2::SP - SAML Service Provider object
 
-=head1 SYNOPSIS
-
-  my $sp = Net::SAML2::SP->new(
-    id   => 'http://localhost:3000',
-    url  => 'http://localhost:3000',
-    cert => 'sign-nopw-cert.pem',
-  );
-
-=head1 METHODS
-
-=cut
 
 use Crypt::OpenSSL::X509;
 use XML::Generator;
 
-=head2 new( ... )
+use Net::SAML2::Binding::POST;
+use Net::SAML2::Binding::Redirect;
+use Net::SAML2::Binding::SOAP;
+use Net::SAML2::Protocol::AuthnRequest;
+use Net::SAML2::Protocol::LogoutRequest;
 
-Constructor. Create an SP object. 
-
-Arguments:
-
- * url    - the base for all SP service URLs
- * id     - the SP's identity URI. 
- * cert   - path to the signing certificate
- * cacert - path to the CA certificate for verification
-
- * org_name         - the SP organisation name
- * org_display_name - the SP organisation display name
- * org_contact      - an SP contact email address
-
-=cut
 
 has 'url'    => (isa => Uri, is => 'ro', required => 1, coerce => 1);
-has 'id'     => (isa => Str, is => 'ro', required => 1);
-has 'cert'   => (isa => Str, is => 'ro', required => 1);
-has 'cacert' => (isa => Str, is => 'ro', required => 1);
+has 'id'     => (isa => 'Str', is => 'ro', required => 1);
+has 'cert'   => (isa => 'Str', is => 'ro', required => 1);
+has 'key'    => (isa => 'Str', is => 'ro', required => 1);
+has 'cacert' => (isa => 'Maybe[Str]', is => 'ro', required => 1);
 
-has 'org_name'         => (isa => Str, is => 'ro', required => 1);
-has 'org_display_name' => (isa => Str, is => 'ro', required => 1);
-has 'org_contact'      => (isa => Str, is => 'ro', required => 1);
+has 'error_url'        => (isa => 'Str', is => 'ro', required => 1);
+has 'slo_url_soap'     => (isa => 'Str', is => 'ro', required => 1);
+has 'slo_url_redirect' => (isa => 'Str', is => 'ro', required => 1);
+has 'slo_url_post'     => (isa => 'Str', is => 'ro', required => 1);
+has 'acs_url_post'     => (isa => 'Str', is => 'ro', required => 1);
+has 'acs_url_artifact' => (isa => 'Str', is => 'ro', required => 1);
+
+has 'org_name'         => (isa => 'Str', is => 'ro', required => 1);
+has 'org_display_name' => (isa => 'Str', is => 'ro', required => 1);
+has 'org_contact'      => (isa => 'Str', is => 'ro', required => 1);
+has 'org_url'          => (isa => 'Str', is => 'ro', required => 0);
+
+has '_cert_text' => (isa => 'Str', is => 'rw', required => 0);
+
+has 'authnreq_signed'         => (isa => 'Bool', is => 'ro', required => 0);
+has 'want_assertions_signed'  => (isa => 'Bool', is => 'ro', required => 0);
 
-has '_cert_text' => (isa => Str, is => 'rw', required => 0);
 
 sub BUILD {
     my ($self) = @_;
@@ -57,38 +48,24 @@ sub BUILD {
     my $text = $cert->as_string;
     $text =~ s/-----[^-]*-----//gm;
     $self->_cert_text($text);
-        
+
     return $self;
 }
 
-=head2 authn_request($destination)
-
-Returns an AuthnRequest object created by this SP, intended for the
-given destination, which should be the identity URI of the IdP.
-
-=cut
 
 sub authn_request {
     my ($self, $destination, $nameid_format) = @_;
-        
+
     my $authnreq = Net::SAML2::Protocol::AuthnRequest->new(
         issueinstant  => DateTime->now,
         issuer        => $self->id,
         destination   => $destination,
         nameid_format => $nameid_format,
     );
-        
+
     return $authnreq;
 }
 
-=head2 logout_request($destination, $nameid, $nameid_format, $session)
-
-Returns a LogoutRequest object created by this SP, intended for the
-given destination, which should be the identity URI of the IdP.
-
-Also requires the nameid (+format) and session to be logged out. 
-
-=cut
 
 sub logout_request {
     my ($self, $destination, $nameid, $nameid_format, $session) = @_;
@@ -104,15 +81,6 @@ sub logout_request {
     return $logout_req;
 }
 
-=head2 logout_response($destination, $status, $response_to)
-
-Returns a LogoutResponse object created by this SP, intended for the
-given destination, which should be the identity URI of the IdP.
-
-Also requires the status and the ID of the corresponding
-LogoutRequest.
-
-=cut
 
 sub logout_response {
     my ($self, $destination, $status, $response_to) = @_;
@@ -128,132 +96,97 @@ sub logout_response {
     return $logout_req;
 }
 
-=head2 artifact_request($destination, $artifact)
-
-Returns an ArtifactResolve request object created by this SP, intended
-for the given destination, which should be the identity URI of the
-IdP.
-
-=cut
 
 sub artifact_request {
     my ($self, $destination, $artifact) = @_;
-        
+
     my $artifact_request = Net::SAML2::Protocol::ArtifactResolve->new(
         issuer       => $self->id,
         destination  => $destination,
         artifact     => $artifact,
         issueinstant => DateTime->now,
     );
-        
+
     return $artifact_request;
 }
 
-=head2 sso_redirect_binding($idp, $param)
-
-Returns a Redirect binding object for this SP, configured against the
-given IDP for Single Sign On. $param specifies the name of the query
-parameter involved - typically SAMLRequest.
-
-=cut
 
 sub sso_redirect_binding {
     my ($self, $idp, $param) = @_;
-        
+
     my $redirect = Net::SAML2::Binding::Redirect->new(
         url   => $idp->sso_url('urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect'),
         cert  => $idp->cert('signing'),
-        key   => $self->cert,
+        key   => $self->key,
         param => $param,
     );
-        
+
     return $redirect;
 }
 
-=head2 slo_redirect_binding
-
-Returns a Redirect binding object for this SP, configured against the
-given IDP for Single Log Out. $param specifies the name of the query
-parameter involved - typically SAMLRequest or SAMLResponse.
-
-=cut
 
 sub slo_redirect_binding {
     my ($self, $idp, $param) = @_;
-        
+
     my $redirect = Net::SAML2::Binding::Redirect->new(
         url   => $idp->slo_url('urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect'),
         cert  => $idp->cert('signing'),
-        key   => $self->cert,
+        key   => $self->key,
         param => $param,
+        sls_force_lcase_url_encoding => $idp->{sls_force_lcase_url_encoding},
+        sls_double_encoded_response => $idp->{sls_double_encoded_response},
     );
-        
     return $redirect;
 }
 
-=head2 soap_binding
-
-Returns a SOAP binding object for this SP, with a destination of the
-given URL and signing certificate.
-
-XXX UA
-
-=cut
 
 sub soap_binding {
     my ($self, $ua, $idp_url, $idp_cert) = @_;
 
     my $soap = Net::SAML2::Binding::SOAP->new(
         ua       => $ua,
-        key      => $self->cert,
+        key      => $self->key,
         cert     => $self->cert,
         url      => $idp_url,
         idp_cert => $idp_cert,
         cacert   => $self->cacert,
     );
-        
+
     return $soap;
 }
 
-=head2 post_binding
-
-Returns a POST binding object for this SP.
-
-=cut
 
 sub post_binding {
     my ($self) = @_;
-        
+
     my $post = Net::SAML2::Binding::POST->new(
         cacert => $self->cacert,
     );
-        
+
     return $post;
 }
 
-=head2 metadata
-
-Returns the metadata XML document for this SP.
-
-=cut
 
 sub metadata {
     my ($self) = @_;
 
+    use Net::SAML2::Util qw/generate_id/;
+
     my $x = XML::Generator->new(':pretty', conformance => 'loose');
     my $md = ['md' => 'urn:oasis:names:tc:SAML:2.0:metadata'];
     my $ds = ['ds' => 'http://www.w3.org/2000/09/xmldsig#'];
 
-    $x->EntityDescriptor(
+    my $metadata = $x->EntityDescriptor(
         $md,
         {
             entityID => $self->id },
         $x->SPSSODescriptor(
             $md,
-            { AuthnRequestsSigned => '1',
-              WantAssertionsSigned => '1',
-              errorURL => $self->url . '/saml/error',
-              protocolSupportEnumeration => 'urn:oasis:names:tc:SAML:2.0:protocol' },
+            { AuthnRequestsSigned => defined($self->authnreq_signed) ? $self->authnreq_signed : '1',
+              WantAssertionsSigned => defined($self->want_assertions_signed) ? $self->want_assertions_signed : '1',
+              errorURL => $self->url . $self->error_url,
+              protocolSupportEnumeration => 'urn:oasis:names:tc:SAML:2.0:protocol',
+              ID => generate_id()},
             $x->KeyDescriptor(
                 $md,
                 {
@@ -272,24 +205,29 @@ sub metadata {
             $x->SingleLogoutService(
                 $md,
                 { Binding => 'urn:oasis:names:tc:SAML:2.0:bindings:SOAP',
-                  Location  => $self->url . '/saml/slo-soap' },
+                  Location  => $self->url . $self->slo_url_soap },
             ),
             $x->SingleLogoutService(
                 $md,
                 { Binding => 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect',
-                  Location  => $self->url . '/saml/sls-redirect-response' },
+                  Location  => $self->url . $self->slo_url_redirect },
+            ),
+            $x->SingleLogoutService(
+                $md,
+                { Binding => 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST',
+                  Location  => $self->url . $self->slo_url_post },
             ),
             $x->AssertionConsumerService(
                 $md,
                 { Binding => 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST',
-                  Location => $self->url . '/saml/consumer-post',
+                  Location => $self->url . $self->acs_url_post,
                   index => '1',
                   isDefault => 'true' },
             ),
             $x->AssertionConsumerService(
                 $md,
                 { Binding => 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Artifact',
-                  Location => $self->url . '/saml/consumer-artifact',
+                  Location => $self->url . $self->acs_url_artifact,
                   index => '2',
                   isDefault => 'false' },
             ),
@@ -312,7 +250,7 @@ sub metadata {
                 $md,
                 {
                     'xml:lang' => 'en' },
-                $self->url
+                defined($self->org_url) ? $self->org_url :$self->url
             )
         ),
         $x->ContactPerson(
@@ -329,6 +267,178 @@ sub metadata {
             ),
         )
     );
+
+    use Net::SAML2::XML::Sig;
+
+    my $signer = Net::SAML2::XML::Sig->new({
+                        key => $self->key,
+                        cert => $self->cert,
+                        sig_hash => 'sha256',
+                        digest_hash => 'sha256',
+                        x509 => 1,
+                });
+
+    # create a signature
+    my $signed = $signer->sign($metadata);
+
+    return $signed;
 }
 
 __PACKAGE__->meta->make_immutable;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Net::SAML2::SP - Net::SAML2::SP - SAML Service Provider object
+
+=head1 VERSION
+
+version 0.44
+
+=head1 SYNOPSIS
+
+  my $sp = Net::SAML2::SP->new(
+    id   => 'http://localhost:3000',
+    url  => 'http://localhost:3000',
+    cert => 'sign-nopw-cert.pem',
+    key => 'sign-nopw-key.pem',
+  );
+
+=head1 NAME
+
+Net::SAML2::SP - SAML Service Provider object
+
+=head1 METHODS
+
+=head2 new( ... )
+
+Constructor. Create an SP object.
+
+Arguments:
+
+=over
+
+=item B<url>
+
+base for all SP service URLs
+
+=item B<id>
+
+SP's identity URI.
+
+=item B<cert>
+
+path to the signing certificate
+
+=item B<key>
+
+path to the private key for the signing certificate
+
+=item B<cacert>
+
+path to the CA certificate for verification
+
+=item B<org_name>
+
+SP organisation name
+
+=item B<org_display_name>
+
+SP organisation display name
+
+=item B<org_contact>
+
+SP contact email address
+
+=item B<org_url>
+
+SP organization url.  This is optional and url will be used as in
+previous versions if this is not provided.
+
+=item B<authnreq_signed>
+
+Specifies in the metadata whether the SP signs the AuthnRequest
+Optional (0 or 1) defaults to 1 (TRUE) if not specified.
+
+=item B<want_assertions_signed>
+
+Specifies in the metadata whether the SP wants the Assertion from
+the IdP to be signed
+Optional (0 or 1) defaults to 1 (TRUE) if not specified.
+
+=back
+
+=head2 BUILD ( hashref of the parameters passed to the constructor )
+
+Called after the object is created to load the cert from a file
+
+=head2 authn_request( $destination, $nameid_format )
+
+Returns an AuthnRequest object created by this SP, intended for the
+given destination, which should be the identity URI of the IdP.
+
+=head2 logout_request( $destination, $nameid, $nameid_format, $session )
+
+Returns a LogoutRequest object created by this SP, intended for the
+given destination, which should be the identity URI of the IdP.
+
+Also requires the nameid (+format) and session to be logged out.
+
+=head2 logout_response( $destination, $status, $response_to )
+
+Returns a LogoutResponse object created by this SP, intended for the
+given destination, which should be the identity URI of the IdP.
+
+Also requires the status and the ID of the corresponding
+LogoutRequest.
+
+=head2 artifact_request( $destination, $artifact )
+
+Returns an ArtifactResolve request object created by this SP, intended
+for the given destination, which should be the identity URI of the
+IdP.
+
+=head2 sso_redirect_binding( $idp, $param )
+
+Returns a Redirect binding object for this SP, configured against the
+given IDP for Single Sign On. $param specifies the name of the query
+parameter involved - typically C<SAMLRequest>.
+
+=head2 slo_redirect_binding( $idp, $param )
+
+Returns a Redirect binding object for this SP, configured against the
+given IDP for Single Log Out. $param specifies the name of the query
+parameter involved - typically C<SAMLRequest> or C<SAMLResponse>.
+
+=head2 soap_binding( $ua, $idp_url, $idp_cert )
+
+Returns a SOAP binding object for this SP, with a destination of the
+given URL and signing certificate.
+
+XXX UA
+
+=head2 post_binding( )
+
+Returns a POST binding object for this SP.
+
+=head2 metadata( )
+
+Returns the metadata XML document for this SP.
+
+=head1 AUTHOR
+
+Chris Andrews  <chrisa@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2021 by Chris Andrews and Others, see the git log.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
